"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const history_1 = require("history");
const utils_1 = require("../utils");
describe('Utils', () => {
    describe('#mapRoutes()', () => {
        it('should return a map containing flattened routes', () => {
            // Given
            const childRoute = {
                name: 'child',
                path: '/parent/child',
            };
            const routes = [
                {
                    name: 'parent',
                    path: '/parent',
                    routes: [childRoute],
                },
            ];
            // When
            const map = utils_1.mapRoutes(routes);
            // Then
            expect(map.get('parent')).toEqual(Object.assign({}, routes[0], { parents: [], regex: /^\/parent\/?$/ }));
            expect(map.get('child')).toEqual(Object.assign({}, childRoute, { parents: ['parent'], regex: /^\/parent\/child\/?$/ }));
        });
        it('should throw if path is given for a route but no name', () => {
            // Given
            const routes = [{ path: '/route-without-name' }];
            // Then
            expect(() => utils_1.mapRoutes(routes)).toThrow(new Error('Route /route-without-name has no name'));
        });
        it('should warn when registering a route twice', () => {
            // Given
            const routes = [{ name: 'duplicate', path: '/route1' }, { name: 'duplicate', path: '/route2' }];
            const consoleMock = jest.spyOn(console, 'warn').mockImplementation(() => { });
            // When
            utils_1.mapRoutes(routes);
            // Then
            expect(consoleMock).toHaveBeenCalledWith('Duplicate definition for route duplicate:\n- /route1\n- /route2');
            consoleMock.mockRestore();
        });
        it('should ignore unnamed route without path', () => {
            // Given
            const routes = [{ component: jest.fn() }];
            // When
            const map = utils_1.mapRoutes(routes);
            // Then
            expect(map.size).toEqual(0);
        });
    });
    describe('#buildRoutingContext()', () => {
        it('should return base context if no history is provided', () => {
            // When
            const context = utils_1.buildRoutingContext([]);
            // Then
            expect(context).toBeInstanceOf(utils_1.BaseRoutingContext);
        });
        it('should return full context if history is provided', () => {
            // When
            const context = utils_1.buildRoutingContext([], history_1.createMemoryHistory());
            // Then
            expect(context).toBeInstanceOf(utils_1.RoutingContext);
        });
    });
    describe('BaseRoutingContext', () => {
        const routesMap = new Map([['test', {
                    path: '/test',
                    regex: /^\/test$/,
                    parents: [],
                }]]);
        describe('#getPath()', () => {
            it('should return path for given route name', () => {
                // Given
                const baseRoutingContext = new utils_1.BaseRoutingContext(routesMap);
                // When
                const path = baseRoutingContext.getPath('test');
                // Then
                expect(path).toEqual('/test');
            });
        });
        describe('#getRoute()', () => {
            it('should return path for given route name', () => {
                // Given
                const baseRoutingContext = new utils_1.BaseRoutingContext(routesMap);
                // When
                const route = baseRoutingContext.getRoute('test');
                // Then
                expect(route).toEqual(routesMap.get('test'));
            });
            it('should throw if route does not exist', () => {
                // Given
                const baseRoutingContext = new utils_1.BaseRoutingContext(routesMap);
                // Then
                expect(() => baseRoutingContext.getRoute('invalid')).toThrow(new Error('Undefined route "invalid"'));
            });
        });
        describe('#match()', () => {
            it('should match route for given pathname', () => {
                // Given
                const baseRoutingContext = new utils_1.BaseRoutingContext(routesMap);
                // When
                const match = baseRoutingContext.match('/test');
                // Then
                expect(match).toEqual(routesMap.get('test'));
            });
        });
    });
    describe('RoutingContext', () => {
        const routesMap = new Map([['test', {
                    path: '/test',
                    regex: /^\/test$/,
                    parents: [],
                }]]);
        describe('#push()', () => {
            it('should call push on history with matching route', () => {
                // Given
                const history = history_1.createMemoryHistory();
                const routingContext = new utils_1.RoutingContext(routesMap, history);
                const historyPushMock = jest.spyOn(history, 'push');
                // When
                routingContext.push('test');
                // Then
                expect(historyPushMock).toHaveBeenCalledWith('/test');
            });
        });
        describe('#replace()', () => {
            it('should call replace on history with matching route', () => {
                // Given
                const history = history_1.createMemoryHistory();
                const routingContext = new utils_1.RoutingContext(routesMap, history);
                const historyReplaceMock = jest.spyOn(history, 'replace');
                // When
                routingContext.replace('test');
                // Then
                expect(historyReplaceMock).toHaveBeenCalledWith('/test');
            });
        });
    });
    describe('#buildRoutePath()', () => {
        it('should return built path for given route', () => {
            // Given
            const map = new Map([['test', {
                        path: '/test/:param',
                        regex: /^\/test$/,
                        parents: [],
                    }]]);
            // When
            const path = utils_1.buildRoutePath(map, 'test', { param: 1234 });
            // Then
            expect(path).toEqual('/test/1234');
        });
        it('should remove trailing slash if optional param not given', () => {
            // Given
            const map = new Map([['test', {
                        path: '/test/:param?',
                        regex: /^\/test$/,
                        parents: [],
                    }]]);
            // When
            const path = utils_1.buildRoutePath(map, 'test');
            // Then
            expect(path).toEqual('/test');
        });
        it('should throw if route does not exist', () => {
            expect(() => utils_1.buildRoutePath(new Map(), 'invalid')).toThrow(new Error('Undefined route "invalid"'));
        });
        it('should throw if route has no path', () => {
            // Given
            const map = new Map([
                ['missingPath', { name: 'missingPath', regex: /^$/, parents: [] }],
            ]);
            // Then
            expect(() => utils_1.buildRoutePath(map, 'missingPath')).toThrow(new Error('Route "missingPath" does not have a path'));
        });
        it('should throw if no value is provided for required param', () => {
            // Given
            const map = new Map([['routeWithParam', {
                        name: 'routeWithParam',
                        path: '/path/with/:param',
                        regex: /^$/,
                        parents: [],
                    }]]);
            // Then
            expect(() => utils_1.buildRoutePath(map, 'routeWithParam'))
                .toThrow(new Error('Missing value for required param "param"'));
        });
    });
});
